//This page extension changes the standard behavior of the Time entry mobile application
import { TimeClassification, GraphApi } from '@showcase/custom-1-api';
import { Project, TimeCategory } from '@sage/x3-project-management-api';
import { MobileTimeEntryInquiryByDate } from '@sage/x3-project-management/build/lib/pages/mobile-time-entry-inquiry-by-date';
import * as ui from '@sage/xtrem-ui';

@ui.decorators.pageExtension<MobileTimeEntryInquiryByDateExtension>({
    extends: '@sage/x3-project-management/MobileTimeEntryInquiryByDate',
})
export class MobileTimeEntryInquiryByDateExtension extends ui.PageExtension<MobileTimeEntryInquiryByDate, GraphApi> {

    @ui.decorators.referenceFieldOverride<MobileTimeEntryInquiryByDateExtension, TimeCategory>({
        //Hide the standard Time category field on the page
        isHidden:true,
    })
    timeCategory: ui.fields.Reference<TimeCategory>;

    @ui.decorators.referenceField<MobileTimeEntryInquiryByDateExtension, TimeClassification>({
        //Add the reference to the new specific Time classification field on the page 
        parent() {
            return this.timeSummaryBlock;
        },
        title: 'Classification',
        node: '@showcase/custom-1/TimeClassification',
        valueField: {code:true},
        helperTextField: {description:true},
        columns: [
            ui.nestedFields.text({ bind: 'code', title: 'Code',}),
            ui.nestedFields.text({ bind: 'description', title: 'Description',}),
        ],
    })
    timeClassification: ui.fields.Reference<TimeClassification>;

    @ui.decorators.referenceFieldOverride<MobileTimeEntryInquiryByDateExtension, Project>({
        //Add the Launching date and Due date columns in the Project selection list, sort the selection by Due date
        columns:[
            ui.nestedFields.text({bind:'launchingDate', title:'Launching date' }), 
            ui.nestedFields.text({bind:'dueDate', title:'Due date' }), 
        ],
        orderBy: {
            dueDate: -1, 
        }
    })
    project: ui.fields.Reference<Project>;

    getSerializedValues() {
        //Change the Time entry description if a Time classification is selected on the page. 
        //This action is performed before sending the payload to the server. It's an override of the original and not a chained execution! 
        const { values } = this.$;
        if ((this.timeClassification.value?.description??'')!=='')
            {values.localizedDescription=`SHOWCASE --> ${this.timeClassification.value?.description}`}
        return values as any;
    }

}

declare module '@sage/x3-project-management/build/lib/pages/mobile-time-entry-inquiry-by-date' {
    interface MobileTimeEntryInquiryByDate extends MobileTimeEntryInquiryByDateExtension {}
}
