import { GraphApi, Project, Task, Operation, OperationAssignment } from '@sage/x3-project-management-api';
import * as ui from '@sage/xtrem-ui';
import { showcase } from '../menu-items/showcase';
import { setApplicativePageCrudActions } from '@sage/x3-system/build/lib/client-functions/applicative-crud-actions';

@ui.decorators.page<ShowcaseProjectManagement, Project>({
    title: 'Project management (Nested grid)',
    subtitle: 'Enter details',
    node: '@sage/x3-project-management/Project',
    objectTypeSingular: 'Project',
    objectTypePlural: 'Projects',
    menuItem: showcase,
    priority: 150,
    idField() {
        return this.id;
    },
    onLoad() {
        this.stepIndicator.value = 'Started'
        setApplicativePageCrudActions({
            page: this,
            isDirty: false,
            save: this.$standardSaveAction,
            cancel: this.$standardCancelAction,
            create: this.$standardNewAction,
        });
    },
    async onDirtyStateUpdated(isDirty: boolean) {
        setApplicativePageCrudActions({
            page: this,
            isDirty,
            save: this.$standardSaveAction,
            cancel: this.$standardCancelAction,
        });
    },
    navigationPanel: {
        isFirstLetterSeparatorHidden: true,
        orderBy: { localizedDescription: 1, },
        listItem: {
            image: ui.nestedFields.image({
                bind: {site:{legalCompany:{logo:{data:true}}}},
                title: 'Company',
                placeholderMode: 'FirstThree',
            }),
            title: ui.nestedFields.text({ bind: 'localizedDescription', title: 'Description'}),
            line2: ui.nestedFields.text({ bind: 'id',  title: 'Code'}),
            line2Right: ui.nestedFields.text({ bind: 'projectStatus',  title: 'Status'}),
            line3: ui.nestedFields.date({ bind: 'startDate',  title: 'Start date'}),
            line3Right: ui.nestedFields.date({ bind: 'endDate',  title: 'End date'}),
            line4: ui.nestedFields.date({ bind: 'dueDate', title: 'Due Date'}),
            line4Right: ui.nestedFields.numeric({ bind: 'duration',  title: 'Duration'}),
            isTemplate: ui.nestedFields.checkbox({ bind: 'isTemplate',  title: 'Template'}),
            progressBar: ui.nestedFields.progress({ bind: 'estimatedProgressPercentage', title: 'Progress' }),
        },
        bulkActions: [
            {
                mutation: 'bulkDelete',
                title: 'Delete',
                buttonType: 'tertiary',
                icon: 'delete',
                isDestructive: true,
            },
        ],
        optionsMenu: [
            { title: 'All', graphQLFilter: {}, },
            { title: 'Template', graphQLFilter: { isTemplate: { _eq: true } }, },
            { title: 'New', graphQLFilter: { projectStatus: { _eq: 'new' } }, },
            { title: 'Launched', graphQLFilter: { projectStatus: { _eq: 'launched' } }, },
            { title: 'Delivered', graphQLFilter: { projectStatus: { _eq: 'delivered' } }, },
            { title: 'Closed', graphQLFilter: { projectStatus: { _eq: 'closed' } }, },
            { title: 'Suspended', graphQLFilter: { projectStatus: { _eq: 'suspended' } }, },
        ],
    },
    headerDropDownActions() {
       return [this.$standardSaveAction, this.$standardCancelAction, this.$standardOpenRecordHistoryAction, ui.MENU_SEPARATOR, this.$standardDuplicateAction] 
    },
})
export class ShowcaseProjectManagement extends ui.Page<GraphApi, Project> {

    @ui.decorators.section<ShowcaseProjectManagement>({
        isTitleHidden: true,
    })
    headerSection: ui.containers.Section;

    @ui.decorators.block<ShowcaseProjectManagement>({
        parent() {
            return this.headerSection;
        },
        isTitleHidden: true,
    })
    headertBlock: ui.containers.Block;

    @ui.decorators.tile<ShowcaseProjectManagement>({
        parent() {
            return this.headerSection;
        },
    })
    tileContainer: ui.containers.Tile;

    @ui.decorators.block<ShowcaseProjectManagement>({
        parent() {
            return this.headerSection;
        },
        title : 'Activty planning'
    })
    taskBlock: ui.containers.Block;

    @ui.decorators.stepSequenceField<ShowcaseProjectManagement>({
        parent() {
            return this.headertBlock;
        },
        isTransient: true,
        options: [
            ui.localize('@showcase/custom-1/pages__ShowcaseProjectManagement__New', 'New'),
            ui.localize('@showcase/custom-1/pages__ShowcaseProjectManagement__Opened', 'Opened'),
            ui.localize('@showcase/custom-1/pages__ShowcaseProjectManagement__Started', 'Started'),
            ui.localize('@showcase/custom-1/pages__ShowcaseProjectManagement__Closed', 'Closed')
        ],
    })
    stepIndicator: ui.fields.StepSequence;

    @ui.decorators.textField<ShowcaseProjectManagement>({
        parent() {
            return this.headertBlock;
        },
        title: '_id',
        isReadOnly: true,
        isHidden: true,
    })
    id: ui.fields.Text;
    
    @ui.decorators.dateField<ShowcaseProjectManagement>({
        parent() {
            return this.tileContainer;
        },
        bind: 'startDate',
        title: 'Start date',
        width: 'medium',
    })
    startdDateTile: ui.fields.Date;

    @ui.decorators.dateField<ShowcaseProjectManagement>({
        parent() {
            return this.tileContainer;
        },
        bind: 'endDate',
        title: 'End date',
        width: 'medium',
    })
    enddDateTile: ui.fields.Date;

    @ui.decorators.dateField<ShowcaseProjectManagement>({
        parent() {
            return this.tileContainer;
        },
        bind: 'dueDate',
        title: 'Due date',
        width: 'medium',
    })
    duedDateTile: ui.fields.Date;

    @ui.decorators.countField<ShowcaseProjectManagement>({
        parent() {
            return this.tileContainer
        },
        bind: 'tasks',
        title: 'Number of tasks',
        width: 'medium',
    })
    lineCountTile: ui.fields.Count;

    @ui.decorators.nestedGridField<ShowcaseProjectManagement, [Task, Operation, OperationAssignment]>({
        parent() {
            return this.taskBlock;
        },
        bind: 'tasks',
        canSelect: false,
        displayMode: ui.fields.TableDisplayMode.compact,
        levels: [
            {
                node: '@sage/x3-project-management/Task',
                childProperty: 'operations',
                orderBy: { _id: +1 },
                columns: [
                 ui.nestedFields.text({ bind: 'code',  title: 'Task'}),
                 ui.nestedFields.text({ bind: 'localizedDescription', title: 'Description'}),
                 ui.nestedFields.date({ bind: 'startDate', title: 'Start date'}),
                 ui.nestedFields.date({ bind: 'endDate', title: 'End date'}),
                 ui.nestedFields.numeric({ bind: 'duration', title: 'Duration'}),
                 ui.nestedFields.text({ bind: 'status', title: 'Status'}),
                 ui.nestedFields.progress({ bind: 'estimatedProgress', title: 'Estimated Progress'}),
                ],
                dropdownActions: [
                    {
                        icon: 'bin',
                        title: 'Delete',
                        isDestructive: true,
                        onClick(_id, rowData) {
                            this.$.showToast(`You clicked Delete ${rowData.code}`);
                        },
                    },
                    {
                        icon: 'tick',
                        title: 'Close',
                        onClick(_id, rowData) {
                            this.$.showToast(`You clicked Close ${rowData.code}`);
                        },
                    },
                    {
                        icon: 'edit',
                        title: 'Edit',
                        onClick(_id, rowData) {
                            this.$.showToast(`You clicked Edit ${rowData.code}`);
                        },
                    },
                    {
                        icon: 'add',
                        title: 'Add',
                        onClick(_id, rowData) {
                            this.$.showToast(`You clicked Add ${rowData.code}`);
                        },
                    },
                ],    
            },
            {
                node: '@sage/x3-project-management/Operation',
                childProperty: 'assignments',
                orderBy: { _id: +1 },
                columns: [
                    ui.nestedFields.text({ bind: 'number',  title: 'Activity'}),
                    ui.nestedFields.text({ bind: 'localizedDescription', title: 'Description'}),
                    ui.nestedFields.reference({
                        node: '@sage/x3-manufacturing-data/WorkCenter',
                        valueField: 'code',
                        bind: 'workCenterLaborResource1',
                        title: 'Ressource',
                        columns: [ui.nestedFields.text({ bind: 'code', title: 'Work center'}),
                                  ui.nestedFields.text({ bind: 'description', title: 'Description'})
                                ],
                    }),
                    ui.nestedFields.date({ bind: 'startDate', title: 'Start date'}),
                    ui.nestedFields.date({ bind: 'endDate', title: 'End date'}),
                    ui.nestedFields.numeric({ bind: 'duration', title: 'Duration'}),
                    ui.nestedFields.text({ bind: 'status', title: 'Status'}),
                    ui.nestedFields.numeric({ bind: 'plannedLoad', title: 'Planned load'}),
                    ui.nestedFields.numeric({ bind: 'consumedLoad', title: 'Consumed load'}),
                    ui.nestedFields.numeric({ bind: 'remainingLoad', title: 'Remaining load'}),
                    ui.nestedFields.reference({
                        node: '@sage/x3-master-data/UnitOfMeasure',
                        valueField: 'localizedDescription',
                        bind: 'unit',
                        title: 'Unit',
                    }),                    
                    ui.nestedFields.technical({    
                        node: '@sage/x3-project-management/Task',
                        bind: 'task',
                        nestedFields: [ui.nestedFields.technical({ bind: '_id'})], 
     
                    }),
                ],
            },
            {
                node: '@sage/x3-project-management/OperationAssignment',
                orderBy: { _id: +1 },
                columns: [
                    ui.nestedFields.image({ bind: {employee:{employee:{image:{data:true}}}}, title: ' ' , isReadOnly: true}),
                    ui.nestedFields.text({ bind: {employee:{employee:{name:true}}}, title: 'Assignee'}),
                    ui.nestedFields.text({ bind: 'localizedDescription', title: 'Description'}),
                    ui.nestedFields.date({ bind: 'startDate', title: 'Start date'}),
                    ui.nestedFields.date({ bind: 'endDate', title: 'End date'}),
                    ui.nestedFields.numeric({ bind: 'plannedLoad', title: 'Planned load'}),
                    ui.nestedFields.numeric({ bind: 'consumedLoad', title: 'Consumed load'}),
                    ui.nestedFields.numeric({ bind: 'remainingLoad', title: 'Remaining load'}),
                    ui.nestedFields.reference({
                        node: '@sage/x3-master-data/UnitOfMeasure',
                        valueField: 'localizedDescription',
                        bind: 'unit',
                        title: 'Unit',
                    }),
                    ui.nestedFields.technical({
                        node: '@sage/x3-project-management/Operation',
                        bind: 'operation',
                        nestedFields: [ui.nestedFields.technical({ bind: '_id' })],
                     
                    }),
                ],
            },
        ],
    })
    tasks: ui.fields.NestedGrid<[Task, Operation, OperationAssignment]>;

}
