import { MiscellaneousTable, User } from '@sage/x3-system-api';
import { setApplicativePageCrudActions } from '@sage/x3-system/build/lib/client-functions/applicative-crud-actions';
import * as ui from '@sage/xtrem-ui';
import { Assessor, GraphApi, TimeClassification } from '@showcase/custom-1-api';
import { NotifyAndWait } from '../client-functions/display';
import { showcase } from '../menu-items/showcase';

@ui.decorators.page<ShowcaseTimeClassificationDialog, TimeClassification>({
    title: 'Time classification (Wizard)',
    subtitle: 'Enter details',
    node: '@showcase/custom-1/TimeClassification',
    objectTypeSingular: 'Time classification',
    objectTypePlural: 'Time classifications',
    mode: 'wizard',
    priority: 130,
    idField() {
        return this.code;
    },
    onLoad() {
        setApplicativePageCrudActions({
            page: this,
            isDirty: false,
            cancel: this.$standardCancelAction,
        });
    },
    async onDirtyStateUpdated(isDirty: boolean) {
        setApplicativePageCrudActions({
            page: this,
            isDirty,
            cancel: this.$standardCancelAction,
        });
    },
    async onClose(isWizardCompleted) {
        if (isWizardCompleted) {
            await this.$.graph.create();
        }
    },
})
export class ShowcaseTimeClassificationDialog extends ui.Page<GraphApi, TimeClassification> {

    _notifier = new NotifyAndWait(this);

    @ui.decorators.section<ShowcaseTimeClassificationDialog>({
        title: 'Classification',
        isTitleHidden: true,
    })
    classification: ui.containers.Section;

    @ui.decorators.section<ShowcaseTimeClassificationDialog>({
        title: 'Details',
        isTitleHidden: true,
    })
    details: ui.containers.Section;

    @ui.decorators.section<ShowcaseTimeClassificationDialog>({
        title: 'Assessors',
        isTitleHidden: true,
    })
    AssessorTable: ui.containers.Section;

    @ui.decorators.block<ShowcaseTimeClassificationDialog>({
        parent(){
            return this.classification;
        },
        title: 'Classification',
        isTitleHidden: true,
    })
    classificationBlock: ui.containers.Block;

    @ui.decorators.block<ShowcaseTimeClassificationDialog>({
        parent(){
            return this.details;
        },
        title: 'Details',
        isTitleHidden: true,
    })
    detailsBlock: ui.containers.Block;

    @ui.decorators.textField<ShowcaseTimeClassificationDialog>({
        title: '_id',
    })
    _id: ui.fields.Text;

    @ui.decorators.textField<ShowcaseTimeClassificationDialog>({
        parent() {
            return this.classificationBlock;
        },
        title: 'Code',
        isMandatory: true,
        validation: /[A-Z]/,
        async onChange() {
            this.code.getNextField(true)?.focus();
        },
    })
    code: ui.fields.Text;

    @ui.decorators.textField<ShowcaseTimeClassificationDialog>({
        parent() {
            return this.classificationBlock;
        },
        title: 'Description',
        isMandatory: true,
        async onChange() {
            this.description.getNextField(true)?.focus();
        },
    })
    description: ui.fields.Text;

    @ui.decorators.selectField<ShowcaseTimeClassificationDialog>({
        parent() {
            return this.classificationBlock;
        },
        isTransient: true,
        title: 'Status',
        helperText: 'Transient field',
        options: [
                ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__draft', 'Draft'),
                ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__validated', 'Validated'),
                ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__controled', 'Controled'),
                ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__approved', 'Approved')
            ],
        infoMessage() {
            if (this.status.value === 'Draft') {
                return 'To be validated';
            }
            return null;
        },
    })
    status: ui.fields.Select;

    @ui.decorators.selectField<ShowcaseTimeClassificationDialog>({
        optionType: '@custom-1/enums/TimeClassificationPriority',
        parent() {
            return this.classificationBlock;
        },
        title: 'Priority',
        isSortedAlphabetically: true,
        helperText: 'Select "Critical"',
        warningMessage() {
            if (this.priority.value === 'critical') {
                return 'Warning message ...';
            }
            return null;
        },
    })
    priority: ui.fields.Select;

    @ui.decorators.radioField<ShowcaseTimeClassificationDialog>({
        parent() {
            return this.classificationBlock;
        },
        title: 'Type',
        isMandatory: true,
        optionType: '@showcase/custom-1/ClassificationType',
    })
    type: ui.fields.Radio;

    @ui.decorators.checkboxField<ShowcaseTimeClassificationDialog>({
        parent() {
            return this.classificationBlock;
        },
        title: 'Active',
    })
    isActive: ui.fields.Checkbox;

    @ui.decorators.referenceField<ShowcaseTimeClassificationDialog, MiscellaneousTable>({
        parent() {
            return this.detailsBlock;
        },
        node: '@sage/x3-system/MiscellaneousTable',
        bind: 'pricing',
        title: 'Pricing',
        valueField: 'code',
        helperTextField: 'description',
        filter: {glossaryId:{glossaryId: 6040},},
        columns: [
            ui.nestedFields.text({ bind: 'code', title: 'Code',}),
            ui.nestedFields.text({ bind: 'description', title: 'Description',}),
        ],
    })
    pricing: ui.fields.Reference<MiscellaneousTable>;

    @ui.decorators.referenceField<ShowcaseTimeClassificationDialog, TimeClassification>({
        parent() {
            return this.detailsBlock;
        },
        node: '@showcase/custom-1/TimeClassification',
        title: 'Origin',
        valueField: 'code',
        helperTextField: 'description',
        filter: { isActive: true },
        columns: [
            ui.nestedFields.technical({ bind: '_id'}),
            ui.nestedFields.text({ bind: 'code', title: 'Code'}),
            ui.nestedFields.text({ bind: 'description', title: 'Description'}),
            ui.nestedFields.text({ bind: 'type', title: 'Type'}),
            ui.nestedFields.text({ bind: 'priority', title: 'Priority'}),
            ui.nestedFields.checkbox({ bind: 'isActive', title: 'Active'}),
        ],
    })
    origin: ui.fields.Reference<TimeClassification>;

    @ui.decorators.dateField<ShowcaseTimeClassificationDialog>({
        parent() {
            return this.detailsBlock;
        },
        title: 'Target date',
    })
    targetDate: ui.fields.Date;

    @ui.decorators.progressField<ShowcaseTimeClassificationDialog>({
        parent() {
            return this.detailsBlock;
        },
        title: 'Progress',
    })
    target: ui.fields.Progress;

    @ui.decorators.buttonField<ShowcaseTimeClassificationDialog>({
        map: () => 'Open dialog',
        isTransient: true,
        parent() {
            return this.detailsBlock;
        },
        async onClick() {
            const options: ui.dialogs.DialogOptions = {
                size: 'medium',
                acceptButton: {
                    text: ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__select-critical', 'Accept'),
                },
                cancelButton: {
                    text: ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__select-cancel', 'Cancel'),
                },
            };
            if (
                await this.$.dialog
                    .confirmation(
                        'info',
                        ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__set-type', 'Set the priority to Critical with warning ?'),
                        ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__select-type', 'Accept or Cancel'),
                        options,
                    )
                    .then(() => true)
                    .catch(() => false)
            ) {
                this.priority.value = 'critical';
                await this._notifier.showAndWait(
                    ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__critical-selected', 'Priority set to Critical with warning'),
                    'success',
                    3000,
                );
            } else {
                this.priority.value = 'major';
                await this._notifier.showAndWait(
                    ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__cancel-selected', 'Action canceled'),
                    'warning',
                    3000,
                );
            }
        },
    })
    confirmationDialogButton: ui.fields.Button;

    @ui.decorators.tableField<ShowcaseTimeClassificationDialog, Assessor>({
        parent() {
            return this.AssessorTable;
        },
        node: '@showcase/custom-1/Assessor',
        bind: 'assessors',
        title: 'Assessors',
        isTitleHidden:true,
        canExport: false,
        canAddNewLine: true,
        canResizeColumns: true,
        isPhantomRowDisabled: false,
        optionsMenu: [
            { title: 'All', graphQLFilter: {} },
            { title: 'Solution-oriented', graphQLFilter: { firstAttribute: {  _eq: 'Solution-oriented'} } },
            { title: 'Leadership', graphQLFilter: { firstAttribute: {  _eq: 'Leadership' } } },
            { title: 'Communication', graphQLFilter: { firstAttribute: {  _eq: 'Communication' } } },
            {
                title: 'Planning and delivering',
                graphQLFilter: { firstAttribute: {  _eq: 'Planning and delivering' } },
            },
        ],
        columns: [
            ui.nestedFields.reference<ShowcaseTimeClassificationDialog, Assessor, User>({
                title: 'Assessor',
                isMandatory: true,
                bind: 'code',
                valueField: 'code',
                node: '@sage/x3-system/User',
                imageField:{image:{data:true}},
                columns: [
                    ui.nestedFields.text({ bind: 'code', title: 'Code', }),
                    ui.nestedFields.text({ bind: 'name', title: 'Name' }),
                    ui.nestedFields.text({ bind: 'email', title: 'Email' }),
                    ui.nestedFields.image({ bind: {image:{data:true}}, title: 'Picture' }),
                ],
            }),
            ui.nestedFields.text({
                title: 'Name',
                bind: {code:{name:true}},
                isReadOnly:true
            }),
            ui.nestedFields.text({
                title: 'Email',
                bind: {code:{email:true}},
                isReadOnly:true
            }),
            ui.nestedFields.text({ bind: 'mainRole', title: 'Main role', }),
            ui.nestedFields.text({ bind: 'firstAttribute', title: 'Skill 1',  }),
            ui.nestedFields.text({ bind: 'secondAttribute', title: 'Skill 2', }),
            ui.nestedFields.text({ bind: 'thirdAttribute', title: 'Skill 3', }),
            ui.nestedFields.text({ bind: 'fourthAttribute', title: 'Skill 4', }),
            ui.nestedFields.checkbox({ bind: 'isActive', title: 'Active', }),
            ui.nestedFields.image({ bind: {code:{image:{data:true}}}, width: 'large', title: 'Picture', isReadOnly: true, isExcludedFromMainField:true}),
        ],
        mobileCard: {
            title: ui.nestedFields.text({
                title: 'Name',
                bind: {code:{name:true}},
            }),
            line2: ui.nestedFields.text({
                title: 'Email',
                bind: {code:{email:true}},
            }),
            line2Right: ui.nestedFields.text({ bind: 'mainRole', title: 'Role', canFilter: false }),
            image: ui.nestedFields.image({ bind: { code: { image: {data:true} } }, }),
        },
        orderBy: {
            code: 1,
        },
        dropdownActions: [
            {
                icon: 'box_arrow_left',
                title: 'Edit',
                async onClick(rowId) {
                    this.assessors.openSidebar(rowId);
                },
            },
        ],
        sidebar: {
            title(_id, recordValue) {
                return recordValue.mainRole;
            },
            headerDropdownActions: [
                {
                    title: 'Do stuff',
                    onClick(_id, rowData) {
                        this.$.showToast(ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationDialog__do-stuff',`It did stuff! {{id}} {{code}}`,{id:_id,code:rowData.code.code}));
                    },
                },
                ui.MENU_SEPARATOR,
                {
                    title: 'Do some other stuff',
                    icon: 'bank_with_card',
                    onClick(_id, rowData) {
                        this.$.showToast(ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationDialog__do-other-stuff',`It did some other stuff! {{id}} {{code}}`,{id:_id, code:rowData.code.code} ),{type: 'warning'});
                    },
                },
                ui.MENU_SEPARATOR,
                {
                    icon: 'delete',
                    title: 'Remove',
                    async onClick(_id) {
                        this.assessors.removeRecord(_id);
                        this.$.showToast(ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationDialog__remove',`Line deletion is not implemented in X3`, {type: 'warning'}));
                    }
                },
            ],
            headerQuickActions: [
                {
                    title: 'Lorem',
                    icon: 'three_boxes',
                    onClick(_id, rowData) {
                        this.$.showToast(ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationDialog__lorem-action-message',`You clicked Lorem {{code}}`,{code:rowData.code.code}));
                    },
                },
                {
                    title: 'Ipsum',
                    icon: 'video',
                    onClick(_id, rowData) {
                        this.$.showToast(ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationDialog__ipsum',`You clicked Ipsum {{code}}`,{code: rowData.code.code}));
                    },
                },
            ],
            layout() {
                return {
                    mainSection: {
                        title: 'Main section',
                        blocks: {
                            mainBlock: {
                                fields: ['code', 'mainRole'],
                            },
                            anotherBlock: {
                                title: 'Skills',
                                fields: ['firstAttribute', 'secondAttribute', 'thirdAttribute', 'fourthAttribute'],
                            },
                        },
                    },
                    anotherSection: {
                        title: 'Another section',
                        blocks: {
                            someRandomBlock: {
                                fields: ['isActive'],
                            },
                        },
                    },
                };
            },
         },
    })
    assessors: ui.fields.Table<Assessor>;

    @ui.decorators.pageAction<ShowcaseTimeClassificationDialog>({
        title: 'Custom add line action',
        icon: 'factory',
        onClick() {
            this.$.showToast(ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationDialog__custom-action','Custom action is triggered!!'));
        },
    })
    customAddLine: ui.PageAction;

    @ui.decorators.pageAction<ShowcaseTimeClassificationDialog>({
        title: 'Disabled add line action',
        icon: 'feedback',
        isDisabled: true,
        onClick() {
            this.$.showToast(ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationDialog__action-disabled','You should not see this because this action is disabled'));
        },
    })
    customAddLineDisabled: ui.PageAction;
}
