import { GraphApi, TimeClassification, Assessor } from '@showcase/custom-1-api';
import * as ui from '@sage/xtrem-ui';
import { NotifyAndWait } from '../client-functions/display';
import { showcase } from '../menu-items/showcase';
import { MiscellaneousTable} from '@sage/x3-system-api';
import { setApplicativePageCrudActions } from '@sage/x3-system/build/lib/client-functions/applicative-crud-actions';

@ui.decorators.page<ShowcaseTimeClassificationPods, TimeClassification>({
    title: 'Time classification (Pods)',
    subtitle: 'Enter details',
    node: '@showcase/custom-1/TimeClassification',
    objectTypeSingular: 'Time classification',
    objectTypePlural: 'Time classifications',
    menuItem: showcase,
    mode: 'tabs',
    createAction() {
        return this.$standardNewAction;
    },
    priority: 120,
    idField() {
        return this.code;
    },
    onLoad() {
        this.stepIndicator.value = 'Validated';
        setApplicativePageCrudActions({
            page: this,
            isDirty: false,
            save: this.$standardSaveAction,
            cancel: this.$standardCancelAction,
            create: this.$standardNewAction,
        });
    },
    async onDirtyStateUpdated(isDirty: boolean) {
        setApplicativePageCrudActions({
            page: this,
            isDirty,
            save: this.$standardSaveAction,
            cancel: this.$standardCancelAction,
        });
    },
    navigationPanel: {
        isFirstLetterSeparatorHidden: true,
        listItem: {
            title: ui.nestedFields.text({ bind: 'code',  title: 'Code'}),
            titleRight: ui.nestedFields.text({ bind: 'description', title: 'Description'}),
            line2: ui.nestedFields.reference({
                title: 'Pricing',
                bind: 'pricing',
                node: '@sage/x3-system/User',
                valueField: { code: true }
            }),
            line2Right: ui.nestedFields.text({ bind: 'priority',  title: 'Priority'}),
            line3: ui.nestedFields.text({ bind: 'type', title: 'Type'}),
            line3Right: ui.nestedFields.date({ bind: 'targetDate',  title: 'Target date'}),
            line4Right: ui.nestedFields.checkbox({ bind: 'isActive', title: 'Active'}),
            progressBar: ui.nestedFields.progress({ bind: 'target', title: 'Progress' }),
        },
    },
    businessActions() {
        return [this.customMutation, this.$standardSaveAction, this.$standardCancelAction, this.refreshPage];
    },
    headerQuickActions() {
        return [this.$standardDeleteAction, this.$standardNewAction]
    },
    headerDropDownActions() {
       return [this.$standardSaveAction, this.$standardCancelAction, this.$standardOpenRecordHistoryAction, ui.MENU_SEPARATOR, this.$standardDuplicateAction] 
   },
    headerSection() {
        return this.headerSection;
    },
})
export class ShowcaseTimeClassificationPods extends ui.Page<GraphApi, TimeClassification> {

    getSerializedValues() {
        const {values} = this.$; 
        //You have the ability here to change or customize the payload.
        //The event is executed before sending the payload to the server. Else, this function can be removed.
        return values; 
    }
    
    _notifier = new NotifyAndWait(this);

    @ui.decorators.section<ShowcaseTimeClassificationPods>({
        isTitleHidden: true,
    })
    headerSection: ui.containers.Section;

    @ui.decorators.block<ShowcaseTimeClassificationPods>({
        parent() {
            return this.headerSection;
        },
        title: 'Classification',
        isTitleHidden: true,
    })
    classification: ui.containers.Block;

    @ui.decorators.tile<ShowcaseTimeClassificationPods>({
        parent() {
            return this.headerSection;
        },
    })
    tileContainer: ui.containers.Tile;

    @ui.decorators.section<ShowcaseTimeClassificationPods>({
        title: 'Assessors',
    })
    tab1: ui.containers.Section;

    @ui.decorators.section<ShowcaseTimeClassificationPods>({
        title: 'Status',
    })
    tab2: ui.containers.Section;

    @ui.decorators.section<ShowcaseTimeClassificationPods>({
        title: 'Details',
    })
    tab3: ui.containers.Section;

    @ui.decorators.block<ShowcaseTimeClassificationPods>({
        parent() {
            return this.tab1;
        },
        width: 'extra-large',
    })
    AssessorPods: ui.containers.Block;

    @ui.decorators.block<ShowcaseTimeClassificationPods>({
        parent() {
            return this.tab2;
        },
    })
    stepIndicatorTab: ui.containers.Block;

    @ui.decorators.block<ShowcaseTimeClassificationPods>({
        parent() {
            return this.tab3;
        },
    })
    details: ui.containers.Block;

    @ui.decorators.stepSequenceField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.classification;
        },
        isTransient: true,
        options: [
            ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__draft', 'Draft'),
            ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__validated', 'Validated'),
            ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__controled', 'Controled'),
            ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__approved', 'Approved')
        ],
    })
    stepIndicator: ui.fields.StepSequence;

    @ui.decorators.textField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.classification;
        },
        title: '_id',
        isReadOnly: true,
        isHidden: true,
    })
    _id: ui.fields.Text;

    @ui.decorators.textField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.classification;
        },
        title: 'Code',
        isMandatory: true,
        validation: /[A-Z]/,
        async onChange() {
            this.code.getNextField(true)?.focus();
        },
    })
    code: ui.fields.Text;

    @ui.decorators.textField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.classification;
        },
        title: 'Description',
        isMandatory: true,
        async onChange() {
            this.description.getNextField(true)?.focus();
        },
    })
    description: ui.fields.Text;

    @ui.decorators.selectField<ShowcaseTimeClassificationPods>({
        optionType: '@custom-1/enums/TimeClassificationPriority',
        parent() {
            return this.classification;
        },
        title: 'Priority',
        isSortedAlphabetically: true,
        helperText: 'Select "Critical"',
        warningMessage() {
            if (this.priority.value === 'critical') {
                return 'Warning message ...';
            }
            return null; 
        },
    })
    priority: ui.fields.Select;

    @ui.decorators.radioField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.classification;
        },
        title: 'Type',
        isMandatory: true,
        optionType: '@showcase/custom-1/ClassificationType',
    })
    type: ui.fields.Radio;

    @ui.decorators.checkboxField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.classification;
        },
        title: 'Active',
    })
    isActive: ui.fields.Checkbox;

    @ui.decorators.textField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.tileContainer;
        },
        bind: 'priority',
        title: 'Priority',
        width: 'medium',
    })
    priorityTile: ui.fields.Text;

    @ui.decorators.dateField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.tileContainer;
        },
        bind: 'targetDate',
        title: 'Target date',
        width: 'medium',
    })
    earliestdDateTile: ui.fields.Date;

    @ui.decorators.countField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.tileContainer;
        },
        bind: 'assessors',
        title: 'Number of assessors',
        width: 'medium',
    })
    lineCountTile: ui.fields.Count;
    
    @ui.decorators.referenceField<ShowcaseTimeClassificationPods, MiscellaneousTable>({
        parent() {
            return this.details;
        },
        node: '@sage/x3-system/MiscellaneousTable',
        bind: 'pricing',
        title: 'Pricing',
        valueField: 'code',
        helperTextField: 'description',
        isReadOnly: true,
        filter:{ glossaryId:{glossaryId: 6040}}, 
        columns: [
            ui.nestedFields.text({ bind: 'code', title: 'Code',}),
            ui.nestedFields.text({ bind: 'description', title: 'Description',}),
        ],
    })
    pricing: ui.fields.Reference<MiscellaneousTable>;

    @ui.decorators.referenceField<ShowcaseTimeClassificationPods, TimeClassification>({
        parent() {
            return this.details;
        },
        node: '@showcase/custom-1/TimeClassification',
        title: 'Origin',
        valueField: 'code',
        helperTextField: 'description',
        size: 'medium',
        filter:{ isActive: true }, 
        columns: [
            ui.nestedFields.technical({ bind: '_id'}),
            ui.nestedFields.text({ bind: 'code', title: 'Code'}),
            ui.nestedFields.text({ bind: 'description', title: 'Description'}),
            ui.nestedFields.text({ bind: 'type', title: 'Type'}),
            ui.nestedFields.text({ bind: 'priority', title: 'Priority'}),
            ui.nestedFields.checkbox({ bind: 'isActive', title: 'Active'}),
        ],
    })
    origin: ui.fields.Reference<TimeClassification>;

    @ui.decorators.dateField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.details;
        },
        title: 'Target date',
    })
    targetDate: ui.fields.Date;

    @ui.decorators.progressField<ShowcaseTimeClassificationPods>({
        parent() {
            return this.details;
        },
        title: 'Progress',
    })
    target: ui.fields.Progress;

    @ui.decorators.buttonField<ShowcaseTimeClassificationPods>({
        map: () => 'Critical',
        isTransient: true,
        parent() {
            return this.classification;
        },
        async onClick() {
            const options: ui.dialogs.DialogOptions = {
                size: 'medium',
                acceptButton: {
                    text: ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__select-critical', 'Accept'),
                },
                cancelButton: {
                    text: ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__select-cancel', 'Cancel'),
                },
                //resolveOnCancel:true can also be used
            };
            if (
                await this.$.dialog
                    .confirmation(
                        'info',
                        ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__set-type', 'Set the priority to Critical?'),
                        ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__select-type', 'Accept or Cancel'),
                        options,
                    )
                    .then(() => true)
                    .catch(() => false)
            ) {
                this.priority.value = 'critical';
                await this._notifier.showAndWait(
                    ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__critical-selected', 'Priority set to Critical with warning'),
                    'success',
                    3000,
                );
            } else {
                this.priority.value = 'major';
                await this._notifier.showAndWait(
                    ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassificationPods__cancel-selected', 'Action canceled'),
                    'warning',
                    3000,
                );
            }
        },
    })
    confirmationDialogButton: ui.fields.Button;

    @ui.decorators.selectionCard<ShowcaseTimeClassificationPods>({
        parent() {
            return this.stepIndicatorTab;
        },
        title: 'Status selection',
        isTransient: true,
        icon() {
            return 'note';
        },
        description() {
            return 'Add description';
        },
        onChange() {
            if (this.statusSelection.value !== this.stepIndicator.value) {
                this.stepIndicator.value = this.statusSelection.value;
            }
        },
        options: ['Draft', 'Validated', 'Controled', 'Approved'],
    })
    statusSelection: ui.fields.SelectionCard;

    @ui.decorators.pageAction<ShowcaseTimeClassificationPods>({
        title: 'Custom add line action',
        icon: 'factory',
        onClick() {
            this.$.showToast('Custom action is triggered!!');
        },
    })
    customAddLine: ui.PageAction;

    @ui.decorators.pageAction<ShowcaseTimeClassificationPods>({
        title: 'Disabled add line action',
        icon: 'feedback',
        isDisabled: true,
        onClick() {
            this.$.showToast('You should not see this because this action is disabled');
        },
    })
    customAddLineDisabled: ui.PageAction;

    @ui.decorators.podCollectionField<ShowcaseTimeClassificationPods, Assessor>({
        parent() {
            return this.AssessorPods;
        },
        node: '@showcase/custom-1/Assessor',
        recordTitle(value, rowData) {
            return rowData?.code.name ? rowData.code.name : '';
        },
        addButtonText: 'Add an assessor',
        canAddRecord: true,
        canRemoveRecord: true,
        recordWidth: 'small',
        bind: 'assessors',
        title: 'Assessors',
        isTitleHidden: true,
        async onRecordAdded(_id, data) {
            this.$.showToast('A new pod was added', { type: 'success' });
        },
        onError(error: string | (Error & { errors: Array<any> })) { 
            ui.console.error('just testing onError works.');
            //But it's always better to let the framework doing the job!
        },
        dropdownActions: [
            {
                icon: 'edit',
                title: 'Edit assessor',
                async onClick(recordId, data) {
                    this.$.dialog.message(
                        'info',
                        'Edit Row Action was clicked',
                        `Assessor: ${data.code} Row: ${recordId}`,);
                },
            },
        ],
        
        columns: [
            ui.nestedFields.image({ bind: {code:{image:{data:true}}}, isReadOnly: true, isExcludedFromMainField:true}),
            ui.nestedFields.reference<ShowcaseTimeClassificationPods, Assessor>({
                title: 'Code',
                isMandatory: true,
                bind: 'code',
                valueField: 'code',
                node: '@sage/x3-system/User',
                imageField:{ image:{data:true}},
                width:'small',
                columns: [
                    ui.nestedFields.text({ bind: 'code', title: 'Code' }),
                    ui.nestedFields.text({ bind: 'name', title: 'Name' }),
                    ui.nestedFields.text({ bind: 'email', title: 'Email' }),
                    ui.nestedFields.image({ bind: {image:{data:true}}, title: 'Picture' }),
                ],
            }),
            ui.nestedFields.text({title: 'Main Role', bind: 'mainRole', width:'small',}),
            ui.nestedFields.text({title: 'Email', bind: {code:{email:true}}, isReadOnly:true,}),
            ui.nestedFields.checkbox({ bind: 'isActive', title: 'Active'}),
            ui.nestedFields.text({title: 'Skill 1', bind: 'firstAttribute', width:'small',}),
            ui.nestedFields.text({title: 'Skill 2', bind: 'secondAttribute', width:'small',}),
            ui.nestedFields.text({title: 'Skill 3', bind: 'thirdAttribute', width:'small',}),
            ui.nestedFields.text({title: 'Skill 4', bind: 'fourthAttribute', width:'small',}),
        ],
    })
    displayAssessors: ui.fields.PodCollection<Assessor>;

    @ui.decorators.pageAction<ShowcaseTimeClassificationPods>({
        title: 'Refresh',
        async onClick() {
            return this.$.router.refresh();
        },
    })
    refreshPage: ui.PageAction;

    private async _callMyGraphQlScript(): Promise<any> {
        const result = await this.$.graph
            .node('@showcase/custom-1/TimeClassification')
            .mutations.customScript(
                { messageOut: true },
                { parameters: { variable1In: '1', variable2In: 1, messageOut: '' } },
            )
            .execute();
        if (result.messageOut === null) {
            //But it's always better to throw an error in the backend than in the frontend 
            throw new Error(
                ui.localize(
                    '@showcase/custom-1/pages__ShowcaseTimeClassificationPods__no-message-returned-by-the-subprogram',
                    'No message returned',
                ),
            );
        } else {
            await this._notifier.showAndWait(result.messageOut, 'success', 9000);
        }
    }

    @ui.decorators.pageAction<ShowcaseTimeClassificationPods>({
        title: 'Custom mutation',
        shortcut: ['f6'],
        async onClick() {
            const result = await this._callMyGraphQlScript();
        },
    })
    customMutation: ui.PageAction;
    
}
