import { GraphApi, TimeClassification, Assessor } from '@showcase/custom-1-api';
import * as ui from '@sage/xtrem-ui';
import { NotifyAndWait } from '../client-functions/display';
import { showcase } from '../menu-items/showcase';
import { MiscellaneousTable} from '@sage/x3-system-api';
import { setApplicativePageCrudActions } from '@sage/x3-system/build/lib/client-functions/applicative-crud-actions';

@ui.decorators.page<ShowcaseTimeClassification, TimeClassification>({
    title: 'Time classification (Sidebar)',
    subtitle: 'Enter details',
    node: '@showcase/custom-1/TimeClassification',
    objectTypeSingular: 'Time classification',
    objectTypePlural: 'Time classifications',
    menuItem: showcase,
    createAction() {
        return this.$standardNewAction;
    },
    priority: 100,
    idField() {
        return this.code;
    },
    onLoad() {
        this.stepIndicator.value = 'Validated';
        setApplicativePageCrudActions({
            page: this,
            isDirty: false,
            save: this.$standardSaveAction,
            cancel: this.$standardCancelAction,
            create: this.$standardNewAction,
        });
    },
    async onDirtyStateUpdated(isDirty: boolean) {
        setApplicativePageCrudActions({
            page: this,
            isDirty,
            save: this.$standardSaveAction,
            cancel: this.$standardCancelAction,
        });
    },
    navigationPanel: {
        menuType: 'toggle',
        isFirstLetterSeparatorHidden: true,
        listItem: {
            title: ui.nestedFields.text({ bind: 'code',  title: 'Code'}),
            titleRight: ui.nestedFields.text({ bind: 'description', title: 'Description'}),
            line2: ui.nestedFields.reference({
                title: 'Pricing',
                bind: 'pricing',
                node: '@sage/x3-system/User',
                valueField: { code: true }
            }),
            line2Right: ui.nestedFields.text({ bind: 'priority',  title: 'Priority'}),
            line3: ui.nestedFields.text({ bind: 'type', title: 'Type'}),
            line3Right: ui.nestedFields.date({ bind: 'targetDate',  title: 'Target date'}),
            line4Right: ui.nestedFields.checkbox({ bind: 'isActive', title: 'Active'}),
            progressBar: ui.nestedFields.progress({ bind: 'target', title: 'Progress' }),
        },
        async optionsMenu(storage, queryParam, username) {
            return [
                {
                    title: ui.localize('@showcase/custom-1/nodes__time_classification__property__tab1', 'Active'),
                    graphQLFilter: { isActive: true },
                },
                {
                    title: ui.localize('@showcase/custom-1/nodes__time_classification__property__tab2', 'Inactive'),
                    graphQLFilter: { isActive: false },
                },
            ];
        },
    },
    businessActions() {
        return [this.customMutation, this.$standardSaveAction, this.$standardCancelAction, this.refreshPage];
    },
    headerQuickActions() {
        return [this.$standardDeleteAction, this.$standardNewAction]
    },
    headerDropDownActions() {
       return [this.$standardSaveAction, this.$standardCancelAction, this.$standardOpenRecordHistoryAction , ui.MENU_SEPARATOR, this.$standardDuplicateAction] 
   },
})
export class ShowcaseTimeClassification extends ui.Page<GraphApi, TimeClassification> {

    getSerializedValues() {
        const {values} = this.$; 
        //You have the ability here to change or customize the payload. 
        //The event is executed before sending the payload to the server. Else, this function can be removed.
        return values; 
    }

    _notifier = new NotifyAndWait(this);

    @ui.decorators.section<ShowcaseTimeClassification>({
        isTitleHidden: true,
    })
    mainSection: ui.containers.Section;

    @ui.decorators.block<ShowcaseTimeClassification>({
        parent() {
            return this.mainSection;
        },
        title: 'Classification',
        isTitleHidden: true,
    })
    classification: ui.containers.Block;

    @ui.decorators.block<ShowcaseTimeClassification>({
        parent() {
            return this.mainSection;
        },
        title: 'Details',
    })
    details: ui.containers.Block;

    @ui.decorators.block<ShowcaseTimeClassification>({
        parent() {
            return this.mainSection;
        },
        title: 'Assessor list',
    })
    AssessorTable: ui.containers.Block;

    @ui.decorators.stepSequenceField<ShowcaseTimeClassification>({
        parent() {
            return this.classification;
        },
        isTransient: true,
        options: [
            ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__draft', 'Draft'),
            ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__validated', 'Validated'),
            ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__controled', 'Controled'),
            ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__approved', 'Approved')
        ],
    })
    stepIndicator: ui.fields.StepSequence;

    @ui.decorators.textField<ShowcaseTimeClassification>({
        parent() {
            return this.classification;
        },
        title: '_id',
        isReadOnly: true,
        isHidden: true,
    })
    _id: ui.fields.Text;

    @ui.decorators.textField<ShowcaseTimeClassification>({
        parent() {
            return this.classification;
        },
        title: 'Code',
        isMandatory: true,
        validation: /[A-Z]/,
        async onChange() {
            this.code.getNextField(true)?.focus();
        },
    })
    code: ui.fields.Text;

    @ui.decorators.textField<ShowcaseTimeClassification>({
        parent() {
            return this.classification;
        },
        title: 'Description',
        isMandatory: true,
        async onChange() {
            this.description.getNextField(true)?.focus();
        },
    })
    description: ui.fields.Text;

    @ui.decorators.selectField<ShowcaseTimeClassification>({
        parent() {
            return this.classification;
        },
        isTransient: true,
        title: 'Status',
        helperText: 'Transient field',
        options: [
                ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__draft', 'Draft'),
                ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__validated', 'Validated'),
                ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__controled', 'Controled'),
                ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__approved', 'Approved')
            ],
        onChange() {
            if (this.status.value !== this.stepIndicator.value) {
                this.stepIndicator.value = this.status.value;
            }
        },
        infoMessage() {
            if (this.status.value === 'Draft') {
                return 'To be validated';
            }
            return null;
        },
    })
    status: ui.fields.Select;

    @ui.decorators.selectField<ShowcaseTimeClassification>({
        optionType: '@custom-1/enums/TimeClassificationPriority',
        parent() {
            return this.classification;
        },
        title: 'Priority',
        isSortedAlphabetically: true,
        helperText: 'Select "Critical"',
        warningMessage() {
            if (this.priority.value === 'critical') {
                return 'Warning message ...';
            }
            return null;
        },
    })
    priority: ui.fields.Select;

    @ui.decorators.radioField<ShowcaseTimeClassification>({
        parent() {
            return this.classification;
        },
        title: 'Type',
        isMandatory: true,
        optionType: '@showcase/custom-1/ClassificationType',
    })
    type: ui.fields.Radio;

    @ui.decorators.checkboxField<ShowcaseTimeClassification>({
        parent() {
            return this.classification;
        },
        title: 'Active',
    })
    isActive: ui.fields.Checkbox;

    @ui.decorators.referenceField<ShowcaseTimeClassification, MiscellaneousTable>({
        parent() {
            return this.details;
        },
        node: '@sage/x3-system/MiscellaneousTable',
        bind: 'pricing',
        title: 'Pricing',
        valueField: 'code',
        helperTextField: 'description',
        filter() {
            return {
                glossaryId:{glossaryId: 6040},
            };
        },
        columns: [
            ui.nestedFields.text({ bind: 'code', title: 'Code',}),
            ui.nestedFields.text({ bind: 'description', title: 'Description',}),
        ],
    })
    pricing: ui.fields.Reference;

    @ui.decorators.referenceField<ShowcaseTimeClassification, TimeClassification>({
        parent() {
            return this.details;
        },
        node: '@showcase/custom-1/TimeClassification',
        title: 'Origin',
        valueField: 'code',
        helperTextField: 'description',
        size: 'medium',
        filter() {
            return { isActive: true };
        },
        columns: [
            ui.nestedFields.technical({ bind: '_id'}),
            ui.nestedFields.text({ bind: 'code', title: 'Code'}),
            ui.nestedFields.text({ bind: 'description', title: 'Description'}),
            ui.nestedFields.text({ bind: 'type', title: 'Type'}),
            ui.nestedFields.text({ bind: 'priority', title: 'Priority'}),
            ui.nestedFields.checkbox({ bind: 'isActive', title: 'Active'}),
        ],
    })
    origin: ui.fields.Reference<TimeClassification>;

    @ui.decorators.dateField<ShowcaseTimeClassification>({
        parent() {
            return this.details;
        },
        title: 'Target date',
    })
    targetDate: ui.fields.Date;

    @ui.decorators.progressField<ShowcaseTimeClassification>({
        parent() {
            return this.details;
        },
        title: 'Progress',
    })
    target: ui.fields.Progress;

    @ui.decorators.buttonField<ShowcaseTimeClassification>({
        map: () => 'Open dialog',
        isTransient: true,
        parent() {
            return this.details;
        },
        async onClick() {
            const options: ui.dialogs.DialogOptions = {
                size: 'medium',
                acceptButton: {
                    text: ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__select-critical', 'Accept'),
                },
                cancelButton: {
                    text: ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__select-cancel', 'Cancel'),
                },
            };
            if (
                await this.$.dialog
                    .confirmation(
                        'info',
                        ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__set-type', 'Set the priority to Critical with warning ?'),
                        ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__select-type', 'Accept or Cancel'),
                        options,
                    )
                    .then(() => true)
                    .catch(() => false)
            ) {
                this.priority.value = 'critical';
                await this._notifier.showAndWait(
                    ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__critical-selected', 'Priority set to Critical with warning'),
                    'success',
                    3000,
                );
            } else {
                this.priority.value = 'major';
                await this._notifier.showAndWait(
                    ui.localize('@showcase/custom-1/pages__ShowcaseTimeClassification__cancel-selected', 'Action canceled'),
                    'warning',
                    3000,
                );
            }
        },
    })
    confirmationDialogButton: ui.fields.Button;

    @ui.decorators.buttonField<ShowcaseTimeClassification>({
        map: () => 'Errors',
        isTransient: true,
        parent() {
            return this.details;
        },
        async onClick() {
            throw new Error(
                //But it's always better to throw an error in the backend than in the frontend 
                ui.localize(
                    '@showcase/custom-1/pages__ShowcaseTimeClassification__error-message-returned',
                    'Custom error message...',
                ),
            );
        },
    })
    errorDialogButton: ui.fields.Button;

    @ui.decorators.tableField<ShowcaseTimeClassification, Assessor>({
        parent() {
            return this.AssessorTable;
        },
        node: '@showcase/custom-1/Assessor',
        bind: 'assessors',
        title: 'Assessors',
        canExport: false,
        canAddNewLine: true,
        canResizeColumns: true,
        isPhantomRowDisabled: false,
        optionsMenu: [
            { title: 'All', graphQLFilter: {} },
            { title: 'Solution-oriented', graphQLFilter: { firstAttribute: {  _eq: 'Solution-oriented'} } },
            { title: 'Leadership', graphQLFilter: { firstAttribute: {  _eq: 'Leadership' } } },
            { title: 'Communication', graphQLFilter: { firstAttribute: {  _eq: 'Communication' } } },
            {
                title: 'Planning and delivering',
                graphQLFilter: { firstAttribute: {  _eq: 'Planning and delivering' } },
            },
        ],
        columns: [
            ui.nestedFields.reference({
                title: 'Assessor',
                isMandatory: true,
                bind: 'code',
                valueField: 'code',
                node: '@sage/x3-system/User',
                imageField:{image:{data:true}},
                columns: [
                    ui.nestedFields.text({ bind: 'code', title: 'Code', }),
                    ui.nestedFields.text({ bind: 'name', title: 'Name' }),
                    ui.nestedFields.text({ bind: 'email', title: 'Email' }),
                    ui.nestedFields.image({ bind: {image:{data:true}}, title: 'Picture' }),
                ],
            }),
            ui.nestedFields.text({
                title: 'Name',
                bind: {code:{name:true}},
                isReadOnly:true
            }),
            ui.nestedFields.text({
                title: 'Email',
                bind: {code:{email:true}},
                isReadOnly:true
            }),
            ui.nestedFields.text({ bind: 'mainRole', title: 'Main role', }), 
            ui.nestedFields.text({ bind: 'firstAttribute', title: 'Skill 1',  }),
            ui.nestedFields.text({ bind: 'secondAttribute', title: 'Skill 2', }),
            ui.nestedFields.text({ bind: 'thirdAttribute', title: 'Skill 3', }),
            ui.nestedFields.text({ bind: 'fourthAttribute', title: 'Skill 4', }),
            ui.nestedFields.checkbox({ bind: 'isActive', title: 'Active', }),
            ui.nestedFields.image({ bind: {code:{image:{data:true}}}, width: 'large', title: 'Picture', isReadOnly: true, isExcludedFromMainField:true}),
        ],
        mobileCard: {
            title: ui.nestedFields.text({
                title: 'Name',
                bind: {code:{name:true}},
            }),
            line2: ui.nestedFields.text({
                title: 'Email',
                bind: {code:{email:true}},
            }),
            line2Right: ui.nestedFields.text({ bind: 'mainRole', title: 'Role', canFilter: false }),
            image: ui.nestedFields.image({ bind: { code: { image: {data:true} } }, }),
        },
        orderBy: {
            code: 1,
        },
        dropdownActions: [
            {
                icon: 'box_arrow_left',
                title: 'Edit',
                async onClick(rowId) {
                    this.assessors.openSidebar(rowId);
                },
            },
        ],
        addItemActions() {
            return [this.customAddLine, this.customAddLineDisabled];
        },
        sidebar: {
            title(_id, recordValue) {
                return recordValue.mainRole;
            },
            headerDropdownActions: [
                {
                    title: 'Do stuff',
                    onClick(_id, rowData) {
                        this.$.showToast(`It did stuff! ${_id} ${rowData.code}`);
                    },
                },
                ui.MENU_SEPARATOR,
                {
                    title: 'Do some other stuff',
                    icon: 'bank_with_card',
                    onClick(_id, rowData) {
                        this.$.showToast(`It did some other stuff! ${_id} ${rowData.code}`, {type: 'warning'});
                    },
                },
                ui.MENU_SEPARATOR,
                {
                    icon: 'delete',
                    title: 'Remove',
                    async onClick(_id) {
                        this.assessors.removeRecord(_id);
                        this.$.showToast(`Line deletetion is not implemented in X3`, {type: 'warning'});
                    }
                },
            ],
            headerQuickActions: [
                {
                    title: 'Lorem',
                    icon: 'three_boxes',
                    onClick(_id, rowData) {
                        this.$.showToast(`You clicked Lorem ${rowData.code}`);
                    },
                },
                {
                    title: 'Ipsum',
                    icon: 'video',
                    onClick(_id, rowData) {
                        this.$.showToast(`You clicked Ipsum ${rowData.code}`);
                    },
                },
            ],
            layout() {
                return {
                    mainSection: {
                        title: 'Main section',
                        blocks: {
                            mainBlock: {
                                fields: ['code', 'mainRole'],
                            },
                            anotherBlock: {
                                title: 'Skills',
                                fields: ['firstAttribute', 'secondAttribute', 'thirdAttribute', 'fourthAttribute'],
                            },
                        },
                    },
                    anotherSection: {
                        title: 'Another section',
                        blocks: {
                            someRandomBlock: {
                                fields: ['isActive'],
                            },
                        },
                    },
                };
            },
         },
    })
    assessors: ui.fields.Table<Assessor>;

    @ui.decorators.pageAction<ShowcaseTimeClassification>({
        title: 'Custom add line action',
        icon: 'factory',
        onClick() {
            this.$.showToast('Custom action is triggered!!');
        },
    })
    customAddLine: ui.PageAction;

    @ui.decorators.pageAction<ShowcaseTimeClassification>({
        title: 'Disabled add line action',
        icon: 'feedback',
        isDisabled: true,
        onClick() {
            this.$.showToast('You should not see this because this action is disabled');
        },
    })
    customAddLineDisabled: ui.PageAction;

    @ui.decorators.pageAction<ShowcaseTimeClassification>({
        title: 'Refresh',
        onClick() {
            return this.$.router.refresh();
        },
    })
    refreshPage: ui.PageAction;

    private async _callMyGraphQlScript(): Promise<any> {
        const result = await this.$.graph
            .node('@showcase/custom-1/TimeClassification')
            .mutations.customScript(
                { messageOut: true },
                { parameters: { variable1In: '1', variable2In: 1, messageOut: '' } },
            )
            .execute();
        if (result.messageOut === null) {
            throw new Error(
                //But it's always better to throw an error in the backend than in the frontend 
                ui.localize(
                    '@showcase/custom-1/pages__ShowcaseTimeClassification__no-message-returned-by-the-subprogram',
                    'No message returned',
                ),
            );
        } else {
            await this._notifier.showAndWait(result.messageOut, 'success', 9000);
        }
    }

    @ui.decorators.pageAction<ShowcaseTimeClassification>({
        title: 'Custom mutation',
        shortcut: ['f6'],
        async onClick() {
            const result = await this._callMyGraphQlScript();
        },
    })
    customMutation: ui.PageAction;
    
}
